﻿namespace XmobiTea.MiniSound.Editor
{
    using UnityEngine;
    using UnityEditor;
    using System.Collections.Generic;
    using XmobiTea.MiniAutoGenerate.Editor;

    [CustomEditor(typeof(SoundManager))]
    public class SoundManagerEditor : Editor
    {
        public override void OnInspectorGUI()
        {
            base.OnInspectorGUI();

            if (GUILayout.Button("Generate SoundConstanceId.cs"))
            {
                GenerateSoundConstanceId();
            }
        }

        [MenuItem("XmobiTea Tools/Sound/Generate SoundConstanceId.cs")]
        public static void GenerateSoundConstanceId()
        {
            var soundSettings = Resources.Load<SoundSettings>(SoundSettings.ResourcesPath);
            if (soundSettings == null)
            {
                Debug.LogError("[Sound] Missing " + SoundSettings.ResourcesPath);

                return;
            }

            var contentBuilder = new System.Text.StringBuilder();
            contentBuilder.AppendLine("// This is Generate File Auto Implement Code form SoundManager.cs");
            contentBuilder.AppendLine();
            contentBuilder.AppendLine("namespace XmobiTea.MiniSound");
            contentBuilder.AppendLine("{");
            contentBuilder.AppendLine("    public partial class SoundConstanceId");
            contentBuilder.AppendLine("    {");

            var nameLst = new List<string>();

            var audioClips = soundSettings.AudioClips;

            for (var i = 0; i < audioClips.Length; i++)
            {
                var audioClip = audioClips[i];
                if (audioClip)
                {
                    var name = AutoGenerate.GetConstanceName(audioClip.name);

                    if (nameLst.Contains(name))
                    {
                        Debug.LogError("[Sound] " + name + " does exists!");
                        continue;
                    }

                    nameLst.Add(name);
                    contentBuilder.AppendLine("        public const string " + name + " = " + "\"" + audioClip.name + "\";");
                }
                else
                {
                    Debug.LogError("[Sound] id " + i + " null when generate!");
                }
            }

            contentBuilder.AppendLine("    }");
            contentBuilder.AppendLine("}");
            AutoGenerate.GenerateFile("SoundConstanceId.cs", contentBuilder, Application.dataPath + "/XmobiTea-constance/Scripts/");
        }
    }
}
