namespace XmobiTea.MiniSound.Editor
{
    using System.IO;
    using UnityEditor;

    using UnityEngine;

    [CustomEditor(typeof(SoundSettings))]
    public class SoundSettingsEditor : Editor
    {
        SerializedProperty musicMixerGroupProp;
        SerializedProperty soundMixerGroupProp;
        SerializedProperty masterMixerProp;
        SerializedProperty audioClipsProp;

        private void OnEnable()
        {
            musicMixerGroupProp = serializedObject.FindProperty("musicMixerGroup");
            soundMixerGroupProp = serializedObject.FindProperty("soundMixerGroup");
            masterMixerProp = serializedObject.FindProperty("masterMixer");
            audioClipsProp = serializedObject.FindProperty("audioClips");
        }

        public override void OnInspectorGUI()
        {
            serializedObject.Update();

            EditorGUILayout.PropertyField(musicMixerGroupProp);
            EditorGUILayout.PropertyField(soundMixerGroupProp);
            EditorGUILayout.PropertyField(masterMixerProp);

            EditorGUILayout.Space(20);

            EditorGUILayout.PropertyField(audioClipsProp);

            if (serializedObject.hasModifiedProperties) serializedObject.ApplyModifiedProperties();

            EditorGUILayout.Space(20);
            
            if (GUILayout.Button("Generate SoundConstanceId.cs"))
            {
                SoundManagerEditor.GenerateSoundConstanceId();
            }
        }

        [MenuItem("XmobiTea Tools/Sound/Open Settings")]
        private static void OpenSettings()
        {
            var soundSettings = GetSettings();

            Selection.SetActiveObjectWithContext(soundSettings, soundSettings);
        }

        public static SoundSettings GetSettings()
        {
            var soundSettings = Resources.Load<SoundSettings>(SoundSettings.ResourcesPath);
            if (soundSettings == null)
            {
                soundSettings = ScriptableObject.CreateInstance<SoundSettings>();

                var dirPath = Application.dataPath + "/Resources";
                if (!Directory.Exists(dirPath)) Directory.CreateDirectory(dirPath);

                var path = "Assets/Resources/" + SoundSettings.ResourcesPath + ".asset";
                AssetDatabase.CreateAsset(soundSettings, path);

                Debug.Log("[Sound] Sound Settings create success at path " + path);
            }

            return soundSettings;
        }
    }
}
