﻿namespace XmobiTea.MiniSound
{
    using XmobiTea.MiniSingleton;
    using System.Collections.Generic;
    using UnityEngine;

    public class SoundManager : Singleton<SoundManager>
    {
        private const string MasterValue = "MasterValue";
        private const string MusicValue = "MusicValue";
        private const string SoundValue = "SoundValue";

        private List<AudioSource> musicAudioSourceLst;

        private List<AudioSource> soundAudioSourceLst;

        private Dictionary<string, AudioClip> soundDic;

        private float lastMusicFloat;

        private float lastSoundFloat;

        private float lastAllFloat;

        private SoundSettings soundSettings;

        protected override void OnInit()
        {
            base.OnInit();

            if (musicAudioSourceLst == null) musicAudioSourceLst = new List<AudioSource>();
            if (soundAudioSourceLst == null) soundAudioSourceLst = new List<AudioSource>();

            if (soundDic == null) soundDic = new Dictionary<string, AudioClip>();

            soundSettings = Resources.Load<SoundSettings>(SoundSettings.ResourcesPath);
            if (soundSettings == null)
            {
                Debug.LogError("[Sound] Missing " + SoundSettings.ResourcesPath);

                return;
            }

            var audioClips = soundSettings.AudioClips;

            for (var i = 0; i < audioClips.Length; i++)
            {
                var audioClip = audioClips[i];
                if (audioClip && !soundDic.ContainsKey(audioClip.name))
                {
                    soundDic.Add(audioClip.name, audioClip);
                }
            }
        }

        public static AudioClip GetAudioClip(string name)
        {
            if (!IsExists) return null;

            // Chưa tồn tại kiểu popup này
            if (!Instance.soundDic.ContainsKey(name))
            {
                Debug.LogError("[Sound] " + name + " does not exits!");
                return null;
            }

            return Instance.soundDic[name];
        }

        public static AudioSource PlayMusic(string name, bool loop)
        {
            if (!IsExists) return null;

            // Chưa tồn tại kiểu popup này
            if (!Instance.soundDic.ContainsKey(name))
            {
                Debug.LogError("[Sound] " + name + " does not exits!");
                return null;
            }

            var clip = Instance.soundDic[name];

            return PlayMusic(clip, loop);
        }

        public static AudioSource PlayMusic(AudioClip clip, bool loop)
        {
            if (!IsExists) return null;

            // Chưa tồn tại kiểu popup này
            if (clip == null)
            {
                Debug.LogError("[Sound] clip null!");
                return null;
            }

            var musicAudioSourceLst = Instance.musicAudioSourceLst;

            for (var i = 0; i < musicAudioSourceLst.Count; i++)
            {
                var musicAudioSource = musicAudioSourceLst[i];

                if (!musicAudioSource.isPlaying)
                {
                    musicAudioSource.clip = clip;

                    musicAudioSource.loop = loop;
                    musicAudioSource.Play();

                    return musicAudioSource;
                }
            }

            if (Instance.soundSettings == null)
            {
                Debug.LogError("[Sound] Missing " + SoundSettings.ResourcesPath);

                return null;
            }

            var instantiateGameObject = Instance.gameObject;
            var audioSource = instantiateGameObject.AddComponent<AudioSource>();
            audioSource.outputAudioMixerGroup = Instance.soundSettings.MusicMixerGroup;
            audioSource.playOnAwake = false;
            audioSource.clip = clip;
            audioSource.loop = loop;
            audioSource.Play();

            musicAudioSourceLst.Add(audioSource);

            return audioSource;
        }

        public static void StopMusic(string name)
        {
            if (!IsExists) return;

            // Chưa tồn tại kiểu âm thanh này
            if (!Instance.soundDic.ContainsKey(name))
            {
                Debug.LogError("[Sound] " + name + " does not exits!");
                return;
            }

            var clip = Instance.soundDic[name];

            StopMusic(clip);
        }

        public static void StopMusic(AudioClip clip)
        {
            if (!IsExists) return;

            if (clip == null)
            {
                Debug.LogError("[Sound] clip null!");
                return;
            }

            var musicAudioSourceLst = Instance.musicAudioSourceLst;

            for (var i = 0; i < musicAudioSourceLst.Count; i++)
            {
                var musicAudioSource = musicAudioSourceLst[i];

                if (musicAudioSource.isPlaying && musicAudioSource.clip != null)
                {
                    if (musicAudioSource.clip.Equals(clip))
                    {
                        musicAudioSource.Stop();
                    }
                }
            }
        }

        public static void StopAllMusic()
        {
            if (!IsExists) return;

            var musicAudioSourceLst = Instance.musicAudioSourceLst;

            for (var i = 0; i < musicAudioSourceLst.Count; i++)
            {
                var musicAudioSource = musicAudioSourceLst[i];
                musicAudioSource.Stop();
            }
        }

        public static AudioSource Play3DMusic(GameObject gameObject, string name, bool loop)
        {
            if (!IsExists) return null;

            // Chưa tồn tại kiểu popup này
            if (!Instance.soundDic.ContainsKey(name))
            {
                Debug.LogError("[Sound] " + name + " does not exits!");
                return null;
            }

            var clip = Instance.soundDic[name];

            return Play3DMusic(gameObject, clip, loop);
        }

        public static AudioSource Play3DMusic(GameObject gameObject, AudioClip clip, bool loop)
        {
            if (!IsExists) return null;

            if (clip == null)
            {
                Debug.LogError("[Sound] clip null!");
                return null;
            }

            if (gameObject == null)
            {
                Debug.LogError("[Sound] gameObject null!");
                return null;
            }

            var musicAudioSources = gameObject.GetComponents<AudioSource>();

            for (var i = 0; i < musicAudioSources.Length; i++)
            {
                var musicAudioSource = musicAudioSources[i];

                if (!musicAudioSource.isPlaying)
                {
                    musicAudioSource.clip = clip;

                    musicAudioSource.loop = loop;
                    musicAudioSource.Play();

                    return musicAudioSource;
                }
            }

            if (Instance.soundSettings == null)
            {
                Debug.LogError("[Sound] Missing " + SoundSettings.ResourcesPath);

                return null;
            }

            var instantiateGameObject = gameObject;
            var audioSource = instantiateGameObject.AddComponent<AudioSource>();
            audioSource.outputAudioMixerGroup = Instance.soundSettings.MusicMixerGroup;
            audioSource.playOnAwake = false;
            audioSource.clip = clip;
            audioSource.loop = loop;
            audioSource.Play();

            return audioSource;
        }

        public static void Stop3DMusic(GameObject gameObject, string name)
        {
            if (!IsExists) return;

            // Chưa tồn tại kiểu âm thanh này
            if (!Instance.soundDic.ContainsKey(name))
            {
                Debug.LogError("[Sound] " + name + " does not exits!");
                return;
            }

            var clip = Instance.soundDic[name];

            Stop3DMusic(gameObject, clip);
        }

        public static void Stop3DMusic(GameObject gameObject, AudioClip clip)
        {
            if (!IsExists) return;

            if (clip == null)
            {
                Debug.LogError("[Sound] clip null!");
                return;
            }

            if (gameObject == null)
            {
                Debug.LogError("[Sound] gameObject null!");
                return;
            }

            var musicAudioSources = gameObject.GetComponents<AudioSource>();

            for (var i = 0; i < musicAudioSources.Length; i++)
            {
                var musicAudioSource = musicAudioSources[i];

                if (musicAudioSource.isPlaying && musicAudioSource.clip != null)
                {
                    if (musicAudioSource.clip.Equals(clip))
                    {
                        musicAudioSource.Stop();
                    }
                }
            }
        }

        public static void StopAll3DMusic(GameObject gameObject)
        {
            if (!IsExists) return;

            if (gameObject == null)
            {
                Debug.LogError("[Sound] gameObject null!");
                return;
            }

            var musicAudioSources = gameObject.GetComponents<AudioSource>();

            for (var i = 0; i < musicAudioSources.Length; i++)
            {
                var musicAudioSource = musicAudioSources[i];
                musicAudioSource.Stop();
            }
        }

        public static void MuteAll()
        {
            if (!IsExists) return;

            if (Instance.soundSettings == null)
            {
                Debug.LogError("[Sound] Missing " + SoundSettings.ResourcesPath);

                return;
            }

            Instance.soundSettings.MasterMixer.SetFloat(MasterValue, -80);
        }

        public static void UnmuteAll()
        {
            if (!IsExists) return;

            if (Instance.soundSettings == null)
            {
                Debug.LogError("[Sound] Missing " + SoundSettings.ResourcesPath);

                return;
            }

            Instance.soundSettings.MasterMixer.SetFloat(MasterValue, Instance.lastAllFloat);
        }

        public static void SetVolumeAll(float value)
        {
            if (!IsExists) return;

            if (value < -80) value = 80;
            else if (value > 0) value = 0;

            Instance.lastAllFloat = value;

            if (Instance.soundSettings == null)
            {
                Debug.LogError("[Sound] Missing " + SoundSettings.ResourcesPath);

                return;
            }

            Instance.soundSettings.MasterMixer.SetFloat(MasterValue, value);
        }

        public static void MuteMusic()
        {
            if (!IsExists) return;

            if (Instance.soundSettings == null)
            {
                Debug.LogError("[Sound] Missing " + SoundSettings.ResourcesPath);

                return;
            }

            Instance.soundSettings.MasterMixer.SetFloat(MusicValue, -80);
        }

        public static bool IsMuteMusic()
        {
            if (!IsExists) return false;

            if (Instance.soundSettings == null)
            {
                Debug.LogError("[Sound] Missing " + SoundSettings.ResourcesPath);

                return false;
            }

            if (Instance.soundSettings.MasterMixer.GetFloat(MusicValue, out float value))
            {
                return value <= -50;
            }

            return false;
        }

        public static void UnmuteMusic()
        {
            if (!IsExists) return;

            if (Instance.soundSettings == null)
            {
                Debug.LogError("[Sound] Missing " + SoundSettings.ResourcesPath);

                return;
            }

            Instance.soundSettings.MasterMixer.SetFloat(MusicValue, Instance.lastMusicFloat);
        }

        public static void SetVolumeMusic(float value)
        {
            if (!IsExists) return;

            if (value < -80) value = 80;
            else if (value > 0) value = 0;

            Instance.lastMusicFloat = value;

            if (Instance.soundSettings == null)
            {
                Debug.LogError("[Sound] Missing " + SoundSettings.ResourcesPath);

                return;
            }

            Instance.soundSettings.MasterMixer.SetFloat(MusicValue, value);
        }

        public static void SetVolumeMusicInt(int intValue)
        {
            if (!IsExists) return;

            if (intValue < 0) intValue = 0;
            else if (intValue > 5) intValue = 5;

            var value = intValue == 0 ? -80f : 0 - (5 - intValue) * 4;
            Instance.lastMusicFloat = value;

            if (Instance.soundSettings == null)
            {
                Debug.LogError("[Sound] Missing " + SoundSettings.ResourcesPath);

                return;
            }

            Instance.soundSettings.MasterMixer.SetFloat(MusicValue, value);
        }

        public static void MuteSound()
        {
            if (!IsExists) return;

            if (Instance.soundSettings == null)
            {
                Debug.LogError("[Sound] Missing " + SoundSettings.ResourcesPath);

                return;
            }

            Instance.soundSettings.MasterMixer.SetFloat(SoundValue, -80);
        }

        public static bool IsMuteSound()
        {
            if (!IsExists) return false;

            if (Instance.soundSettings == null)
            {
                Debug.LogError("[Sound] Missing " + SoundSettings.ResourcesPath);

                return false;
            }

            if (Instance.soundSettings.MasterMixer.GetFloat(SoundValue, out float value))
            {
                return value <= -50;
            }

            return false;
        }

        public static void UnmuteSound()
        {
            if (!IsExists) return;

            if (Instance.soundSettings == null)
            {
                Debug.LogError("[Sound] Missing " + SoundSettings.ResourcesPath);

                return;
            }

            Instance.soundSettings.MasterMixer.SetFloat(SoundValue, Instance.lastSoundFloat);
        }

        public static void SetVolumeSound(float value)
        {
            if (!IsExists) return;

            if (value < -80) value = 80;
            else if (value > 0) value = 0;

            Instance.lastSoundFloat = value;

            if (Instance.soundSettings == null)
            {
                Debug.LogError("[Sound] Missing " + SoundSettings.ResourcesPath);

                return;
            }

            Instance.soundSettings.MasterMixer.SetFloat(SoundValue, value);
        }

        public static void SetVolumeSoundInt(int intValue)
        {
            if (intValue < 0) intValue = 0;
            else if (intValue > 5) intValue = 5;

            var value = intValue == 0 ? -80f : 0 - (5 - intValue) * 4;
            Instance.lastSoundFloat = value;

            if (Instance.soundSettings == null)
            {
                Debug.LogError("[Sound] Missing " + SoundSettings.ResourcesPath);

                return;
            }

            Instance.soundSettings.MasterMixer.SetFloat(SoundValue, value);
        }

        public static AudioSource PlaySound(string name)
        {
            if (!IsExists) return null;

            // Chưa tồn tại kiểu âm thanh này
            if (!Instance.soundDic.ContainsKey(name))
            {
                Debug.LogError("[Sound] " + name + " does not exits!");
                return null;
            }

            var clip = Instance.soundDic[name];

            return PlaySound(clip);
        }

        public static AudioSource PlaySound(AudioClip clip)
        {
            if (!IsExists) return null;

            if (clip == null)
            {
                Debug.LogError("[Sound] clip null!");
                return null;
            }

            var soundAudioSourceLst = Instance.soundAudioSourceLst;

            for (var i = 0; i < soundAudioSourceLst.Count; i++)
            {
                var soundAudioSource = soundAudioSourceLst[i];

                if (!soundAudioSource.isPlaying)
                {
                    soundAudioSource.clip = clip;
                    soundAudioSource.Play();

                    return soundAudioSource;
                }
            }

            if (Instance.soundSettings == null)
            {
                Debug.LogError("[Sound] Missing " + SoundSettings.ResourcesPath);

                return null;
            }

            var instantiateGameObject = Instance.gameObject;
            var audioSource = instantiateGameObject.AddComponent<AudioSource>();
            audioSource.outputAudioMixerGroup = Instance.soundSettings.SoundMixerGroup;
            audioSource.playOnAwake = false;
            audioSource.clip = clip;

            audioSource.Play();

            soundAudioSourceLst.Add(audioSource);

            return audioSource;
        }

        public static void StopSound(string name)
        {
            if (!IsExists) return;

            // Chưa tồn tại kiểu âm thanh này
            if (!Instance.soundDic.ContainsKey(name))
            {
                Debug.LogError("[Sound] " + name + " does not exits!");
                return;
            }

            var clip = Instance.soundDic[name];

            StopSound(clip);
        }

        public static void StopSound(AudioClip clip)
        {
            if (!IsExists) return;

            if (clip == null)
            {
                Debug.LogError("[Sound] clip null!");
                return;
            }

            var soundAudioSourceLst = Instance.soundAudioSourceLst;

            for (var i = 0; i < soundAudioSourceLst.Count; i++)
            {
                var soundAudioSource = soundAudioSourceLst[i];

                if (soundAudioSource.isPlaying && soundAudioSource.clip != null)
                {
                    if (soundAudioSource.clip.Equals(clip))
                    {
                        soundAudioSource.Stop();
                    }
                }
            }
        }

        public static void StopAllSound()
        {
            if (!IsExists) return;

            var soundAudioSourceLst = Instance.soundAudioSourceLst;

            for (var i = 0; i < soundAudioSourceLst.Count; i++)
            {
                var soundAudioSource = soundAudioSourceLst[i];
                soundAudioSource.Stop();
            }
        }

        public static AudioSource Play3DSound(GameObject gameObject, string name)
        {
            if (!IsExists) return null;

            // Chưa tồn tại kiểu âm thanh này
            if (!Instance.soundDic.ContainsKey(name))
            {
                Debug.LogError("[Sound] " + name + " does not exits!");
                return null;
            }

            var clip = Instance.soundDic[name];

            return Play3DSound(gameObject, clip);
        }

        public static AudioSource Play3DSound(GameObject gameObject, AudioClip clip)
        {
            if (!IsExists) return null;

            if (clip == null)
            {
                Debug.LogError("[Sound] clip null!");
                return null;
            }

            if (gameObject == null)
            {
                Debug.LogError("[Sound] gameObject null!");
                return null;
            }

            var soundAudioSources = gameObject.GetComponents<AudioSource>();

            for (var i = 0; i < soundAudioSources.Length; i++)
            {
                var soundAudioSource = soundAudioSources[i];

                if (!soundAudioSource.isPlaying)
                {
                    soundAudioSource.clip = clip;
                    soundAudioSource.Play();

                    return soundAudioSource;
                }
            }

            if (Instance.soundSettings == null)
            {
                Debug.LogError("[Sound] Missing " + SoundSettings.ResourcesPath);

                return null;
            }

            var instantiateGameObject = gameObject;
            var audioSource = instantiateGameObject.AddComponent<AudioSource>();
            audioSource.outputAudioMixerGroup = Instance.soundSettings.SoundMixerGroup;
            audioSource.playOnAwake = false;
            audioSource.clip = clip;

            audioSource.Play();

            return audioSource;
        }

        public static void Stop3DSound(GameObject gameObject, string name)
        {
            if (!IsExists) return;

            // Chưa tồn tại kiểu âm thanh này
            if (!Instance.soundDic.ContainsKey(name))
            {
                Debug.LogError("[Sound] " + name + " does not exits!");
                return;
            }

            var clip = Instance.soundDic[name];

            Stop3DSound(gameObject, clip);
        }

        public static void Stop3DSound(GameObject gameObject, AudioClip clip)
        {
            if (!IsExists) return;

            if (clip == null)
            {
                Debug.LogError("[Sound] clip null!");
                return;
            }

            if (gameObject == null)
            {
                Debug.LogError("[Sound] gameObject null!");
                return;
            }

            var soundAudioSources = gameObject.GetComponents<AudioSource>();

            for (var i = 0; i < soundAudioSources.Length; i++)
            {
                var soundAudioSource = soundAudioSources[i];

                if (soundAudioSource.isPlaying && soundAudioSource.clip != null)
                {
                    if (soundAudioSource.clip.Equals(clip))
                    {
                        soundAudioSource.Stop();
                    }
                }
            }
        }

        public static void StopAll3DSound(GameObject gameObject)
        {
            if (!IsExists) return;

            if (gameObject == null)
            {
                Debug.LogError("[Sound] gameObject null!");
                return;
            }

            var soundAudioSources = gameObject.GetComponents<AudioSource>();

            for (var i = 0; i < soundAudioSources.Length; i++)
            {
                var soundAudioSource = soundAudioSources[i];
                soundAudioSource.Stop();
            }
        }
    }
}
